package org.fuseri.confidentialclient;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.fuseri.model.dto.user.UserCreateDto;
import org.fuseri.model.dto.user.UserDto;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.http.MediaType;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.core.Authentication;
import org.springframework.security.oauth2.client.authentication.OAuth2AuthenticationToken;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.SavedRequestAwareAuthenticationSuccessHandler;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;

import java.io.IOException;

@SpringBootApplication
public class ConfidentialClientApplication {

    public static void main(String[] args) {
        SpringApplication.run(ConfidentialClientApplication.class, args);
    }

    private static String asJsonString(final Object obj) throws JsonProcessingException {
        return new ObjectMapper().writeValueAsString(obj);
    }

    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity httpSecurity) throws Exception {
        httpSecurity
                .authorizeHttpRequests(x -> x
                        // allow anonymous access to listed URLs
                        .requestMatchers("/", "/error", "/robots.txt", "/style.css", "/favicon.ico", "/webjars/**").permitAll()
                        // all other requests must be authenticated
                        .anyRequest().authenticated()
                )
                .oauth2Login(x -> x
                        // our custom handler for successful logins
                        .successHandler(authenticationSuccessHandler())
                )
        ;
        return httpSecurity.build();
    }
        @Bean
        public AuthenticationSuccessHandler authenticationSuccessHandler() {
            return new SavedRequestAwareAuthenticationSuccessHandler() {
                @Override
                public void onAuthenticationSuccess(HttpServletRequest req, HttpServletResponse res, Authentication auth) throws ServletException, IOException {


                    var a= res.getHeaderNames();
                    for (var name : a) {
                        System.out.println(res.getHeader(name));
                    }

                    System.out.println("got here");
                    if (auth instanceof OAuth2AuthenticationToken token
                            && token.getPrincipal() instanceof OidcUser user) {
                        var createDto = new UserCreateDto();
                        createDto.setLastName(user.getFamilyName());
                        createDto.setFirstName(user.getGivenName());
                        createDto.setEmail(user.getEmail());
                        createDto.setUsername(user.getPreferredUsername());

                        //var result = WebClient.builder().baseUrl("http://localhost:8081/users/register").build().post()
                                //.contentType(MediaType.APPLICATION_JSON).body(BodyInserters.fromValue(createDto))
                                //.retrieve();


                        //var out =  result.bodyToMono(UserDto.class).block();
                        //System.out.println(out);
                    }
                    super.onAuthenticationSuccess(req, res, auth);
                }
            };
        }



}
